<?php

define(       "DB_USERNAME", "public"           );
define(       "DB_PASSWORD", "public"           );

define(     "HOST_NAME", gethostname()                                       );
define( "DOCUMENT_ROOT", "/local/served/api/WEB_API_BRIDGE"                  );
define(          "BASE", DOCUMENT_ROOT . "/WebAPIBridge/latest/webapibridge" );
define(  "CONTENT_PATH", BASE . "/share/content"                             );
define(    "PAGES_PATH", BASE . "/share/pages"                               );
define(   "SERVER_NAME", $_SERVER["SERVER_NAME"]                             );
define(      "APP_NAME", $_SERVER["SERVER_NAME"]                             );

if ( FALSE != strpos( SERVER_NAME, ".local" ) )
{
	define(   "DEBUG", FALSE );
	define( "VERBOSE", FALSE );
	define( "SELECTS", TRUE  );
}

set_include_path( BASE . "/dep/libpagecentric/source/php:" . BASE . "/source/php" );

include_once( BASE . "/dep/libpagecentric/lib/autoload.php" );
include_once( BASE . "/lib/autoload.php" );

Page::initialise();

//if ( defined( "VERBOSE" ) && VERBOSE )
IncreaseCounter();

DefineDatabaseHostNames();
DefineDatabaseName     ();
DefineDatabaseVersion  ();


function DefineDatabaseHostNames()
{
	$rdb_domain = "";

	if ( FALSE != strpos( SERVER_NAME, ".local" ) )
	{
		$rdb_domain = "local-db." . str_replace( '.local', '', SERVER_NAME );

        error_log( "Resolving database host for: $rdb_domain" );
	}
	else
	{
		$rdb_domain = "db." . SERVER_NAME;
	}

	if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "RDB Domain: $rdb_domain" );

	$text = null;

    if ( ! apcu_exists( $rdb_domain ) )
    {
        $records = dns_get_record( $rdb_domain, DNS_TXT );
        $n       = count( $records );
        $dbinfo  = ExtractDBInfoFromDNSRecords( $rdb_domain, $records );

        apcu_add( $rdb_domain, $dbinfo );
        error_log( "Caching: $rdb_domain" );

        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Returned $n records" );
    }
    else
    {
        $dbinfo = apcu_fetch( $rdb_domain );

        error_log( "Cached: $rdb_domain" );
    }

    if ( $dbinfo->dbname )
    {
        define( "DB", $dbinfo->dbname );
    }

    switch ( count( $dbinfo->hosts ) )
    {
    case 0:
        define( "DB_HOSTNAME", "localhost" );
        define(    "DB_HOSTS", "localhost" );

        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "DB_HOSTNAME: localhost" );
        break;

    case 1:
        define( "DB_HOSTNAME", $dbinfo->hosts[0] );
        define(    "DB_HOSTS", $dbinfo->hosts[0] );

        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "DB_HOSTNAME: " . $dbinfo->hosts[0] );
        break;

    default:
        define( "DB_HOSTNAME1", $dbinfo->hosts[0]  );
        define( "DB_HOSTNAME2", $dbinfo->hosts[1]  );
        define(     "DB_HOSTS", $dbinfo->hosts[0] . "," . $dbinfo->hosts[1] );

        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "DB_HOSTNAME1: " . $dbinfo->hosts[0] );
        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "DB_HOSTNAME2: " . $dbinfo->hosts[1] );
    }

    error_log( "DB_HOSTNAME: " . DB_HOSTNAME );
    error_log(    "DB_HOSTS: " . DB_HOSTS );

}

function ExtractDBInfoFromDNSRecords( $rdb_domain, $records )
{
    $dbname = null;
    $hosts  = array();

    foreach ( $records as $record )
    {
        $host    = array_key_exists(    "host", $record ) ? $record[   "host"] : "";
        $class   = array_key_exists(   "class", $record ) ? $record[  "class"] : "";
        $ttl     = array_key_exists(     "ttl", $record ) ? $record[    "ttl"] : "";
        $type    = array_key_exists(    "type", $record ) ? $record[   "type"] : "";
        $txt     = array_key_exists(     "txt", $record ) ? $record[    "txt"] : "";

        if ( $host == $rdb_domain )
        {
            $bits = explode( "@", $txt );
            
            switch ( count( $bits ) )
            {
            case 2:
                $dbname = $bits[0];
                $hosts  = explode( ",", $bits[1] );
                break;
            
            case 1:
                $hosts  = explode( ",", $bits[0] );
                break;
            }
            break;
        }
    }

    $dbinfo = (object) array( "dbname" => $dbname, "hosts" => $hosts, "count" => 0 );

    return $dbinfo;
}

function DefineDatabaseName()
{
	if ( ! defined( "DB" ) )
	{
		$server_name = str_replace( "www.", "",  SERVER_NAME );
		$server_name = str_replace( "api-", "", $server_name );
		$server_name = str_replace( "api.", "", $server_name );
		$bits        = explode( ".", $server_name );

		$n = count( $bits );

		for ( $i = 0; $i < $n; $i++ )
		{
			if ( ! IsTLD( $bits[$i], $i ) )
			{
				$db = $bits[$i];
				break;
			}
		}

		if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Database prefix: $db" );

		define( "DB", $db );
	}
}

function DefineDatabaseVersion()
{
	$db_version = "";

	if ( ! defined( "DB" ) || ("" == DB) )
	{
		error_log( "DB not defined!!!" );
	}
	else
	{
		if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "DB prefix: " . DB );

		$result = \replicantdb\ReplicantDB::CallInfo( "SHOW DATABASES", null );

		if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Got result" );


		//error_log( var_dump( $result, TRUE ) );

		if ( "ERROR" == $result->status )
		{
			$hostname = $result->hostname;

			error_log( "Could not connect to database on $hostname: " . $result->error );
		}
		else
		if ( ("OK" == $result->status) && count( $result->results ) )
		{
			$db_name = "";
		
			foreach( $result->results as $tuple )
			{
				$tmp = array_get( $tuple, "Database" );

				//error_log( "Checking: $tmp" );
                //
                //  health000
                //  healthcare000

				if ( string_hasPrefix( $tmp, DB ) )
				{
					$db_name = $tmp;
				}
			}
			$db_version = $db_name ? str_replace( DB, "", $db_name ) : $db_version;

			if ( $db_name )
			{
				if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Found: $db_name" );
			}
			else
			{
				error_log( "Could not find database for: " . DB );
			}
		}
	}
	
	define( "DB_VERSION", $db_version );
}

function IsTLD( $domain_part, $level = 1 )
{
	switch ( $level )
	{
	case "1":
		switch ( $domain_part )
		{
		case "au":
		case "com":
		case "net";
		case "org";
		case "co":
		case "info":
		case "xyz":
		case "local":
			return true;
			
		default:
			return false;
		}
		break;

	case "2":
		switch ( $domain_part )
		{
		case "com":
		case "net";
		case "org";
		case "id":
		case "conf":
			return true;
			
		default:
			return false;
		}
		break;
	
	default:
		return false;
	}
}

function IncreaseCounter()
{
    if ( apcu_exists( "counter" ) )
    {
        apcu_inc( "counter" );
    }
    else
    {
        apcu_add( "counter", 1 );
    }

    error_log( "---- Start of request (" . apcu_fetch( "counter" ) . ")----" );
}

function string_hasPrefix( $string, $prefix )
{
    $tmp = str_replace( $prefix, '', $string );

    return (string_startsWith( $string, $prefix ) && is_numeric( $tmp ));
}

?>
