#!/bin/bash
#
#   This script will, for each directory in the Dropspace sites directory,
#   create an Apache2 configuration file called 'DROPSPACE.<directory name>.conf',
#   which is a copy of the 'DROPSPACE.TEMPLATE.conf' file, that has each occurrance
#   of the pattern %domain_name% replaced with <directory name>.
#
#   It needs to know:
#
#   1)  Where the Document root is - 'sites' is directly under it, and
#   2)  The location of the Apache configuration directory (/etc/apache2 by default).

function usage()
{
    echo "Usage: generate_apache_configurations.sh <Document root> [Apache config dir]"
    exit -1
}

#
#   Utility functions
#

function scriptname()
{
    echo $(readlink -f $0)
}

function main()
{
    local document_root=$1
    local apache_config_dir=$2

    if [ ! -d "$document_root" ]
    then

        echo "Invalid Document Root dir: $document_root"
        usage

    else

        if [ -z "$apache_config_dir" ]
        then

            echo "Using default Apache config dir"
            apache_config_dir="/etc/apache2"

        elif [ ! -d "$apache_config_dir" ]
        then

            echo "Invalid Apache config dir"
            usage

        fi

   echo process_subdirectories "$document_root" "$apache_config_dir"
        process_subdirectories "$document_root" "$apache_config_dir"

    fi
}

function process_subdirectories()
{
    local script_name=`scriptname`
    local script_dir=`dirname "$script_name"`
    local libexec_dir=`dirname "$script_dir"`
    local dropspace_dir=`dirname "$libexec_dir"`
    local latest_dir=`dirname "$dropspace_dir"`
    local DropSpace_dir=`dirname "$latest_dir"`

    local apache_template_dir="$dropspace_dir/configuration/apache2"
    local apache_template_file="$apache_template_dir/DROPSPACE.TEMPLATE.conf"

    local document_root=$1
    if [ -z "$document_root" ]
    then

        document_root=`dirname "$DropSpace_dir"`

    elif [ ! -d "$document_root" ]
    then

        echo "Invalid Document root"
        usage

    fi

    local apache_config_dir=$2
    local target="$apache_config_dir/sites-enabled"

    if [ ! -f "$apache_template_file" ]
    then

        echo "Internal error: could not find Apache configuration template file at: $apache_template_file"
        exit

    else

        echo `date`

        for domain_name in $( ls "$document_root/sites" ); do

            if [ ! -f "$target/DROPSPACE.$domain_name.conf" ]
            then

                echo "Creating: $target/DROPSPACE.$domain_name.conf"
                sed "s|%document_root%|$document_root|g" "$apache_template_dir/DROPSPACE.TEMPLATE.conf" | sed "s|%domain_name%|$domain_name|g" > "$target/DROPSPACE.$domain_name.conf"

            fi

        done
    fi
}

if [ $# -lt 1 ]
then

    usage

else

    main $@

fi
